function [phase , squ_abs_val] = analyse_complex(input_Z,norm_factor);
% ANALYSE_COMPLEX       Phase and squared absolute value of an array of
%                       complex numbers
%                       
%   [phase , squ_abs_val] = analyse_complex(input_Z)
%      squ_abs_val ... Squared absolute value of the elements of input_Z
%      phase       ... The corresponding normalised phase angle 0 <= phi <= 1
%
%   [phase , squ_abs_val] = analyse_complex(input_Z , norm_factor);
%      squ_abs_val ... Squared absolute value of the elements of input_Z
%                      divided by norm_factor
%      phase       ... As above

if nargin < 1
  error('No input-function');
elseif nargin < 2
  norm_factor = 1
end

squ_abs_val = zeros(size(input_Z));         %preallocating memory
squ_abs_val = input_Z .* conj(input_Z);     %calculating the absolute value
if norm_factor ~= 1                         %if necessary normalising the result
  squ_abs_val = squ_abs_val / norm_factor; 
end

phase = zeros(size(input_Z));                  %preallocating memory
phase = atan2(imag(input_Z) , real(input_Z));

%atan2 returns values between -pi and  pi but we want them between 0 and 2 pi
index_mat = phase < 0;                       %searching negative values
phase(index_mat) =  2*pi + phase(index_mat); %adding them to 2pi 
phase = phase / (2*pi);                      %normalising them
