function [r,g,b] = hls2rgb(h,l,s)  
  
  if nargin == 1
    if ndims(h) == 2 % colormap
      if size(h,2) == 3
	status = 'c';
	hls = h;
	h = hls(:,1);
	l = hls(:,2);
	s = hls(:,3);
      else
	error('Colormap must have size nx3 ');
      end
    elseif ndims(h) == 3 % image
      if size(h,3) == 3;
	status = 'i';
	im  = h;
	ims = size(im);
	h = reshape(im(:,:,1),[],1);
	l = reshape(im(:,:,2),[],1);
	s = reshape(im(:,:,3),[],1);
	hls = [h,l,s];
      else
	error('Image must have dimension nxmx3 ');
      end
    else
      error('Dimension of input is wrong ');
    end
  elseif nargin == 3
    rs = size(h);
    status = 'v';
    hls = [h(:),l(:),s(:)];
  else
    error('Number of input arguments wrong ');
  end
  % input check
  if any(h<0 | h>1), error('h out of range [0,1]'); end
  if any(l<0 | l>1), error('l out of range [0,1]'); end
  if any(s<0 | s>1), error('s out of range [0,1]'); end
  
  
  % prep
  r = zeros(size(h));
  g = zeros(size(h));
  b = zeros(size(h));
    
  m2 = zeros(size(h)); 
  ll = l <= 0.5;
  m2(ll) = l(ll) .* ( 1 + s(ll) );
  ll = l >  0.5;
  m2(ll) = l(ll) + s(ll) - l(ll) .* s(ll);
  m1 = 2*l - m2;
  
  % achromatic case
  ls = s == 0;
  r(ls) = l(ls);
  g(ls) = l(ls);
  b(ls) = l(ls);
  % chromatic case
  ls = ~ls;
  r(ls) = rgbval( m1(ls), m2(ls), h(ls)+1/3 );
  g(ls) = rgbval( m1(ls), m2(ls), h(ls) );
  b(ls) = rgbval( m1(ls), m2(ls), h(ls)-1/3 );
  
  % output
  switch status
   case 'c'
    r = [r,g,b];
    g = [];
    b = [];
   case 'i'
    im(:,:,1) = reshape(r,ims(1:2));
    im(:,:,2) = reshape(g,ims(1:2));
    im(:,:,3) = reshape(b,ims(1:2));
    r = im;
    g = [];
    b = [];
   case 'v'
    r = reshape(r,rs);    
    g = reshape(g,rs);
    b = reshape(b,rs);
  end
  
function v=rgbval(m1,m2,h)
  
  v = zeros(size(m1));
  h(h<0) = h(h<0)+1;
  h(h>1) = h(h>1)-1;
  
  lh = h <  1/6;
  v(lh) = m1(lh) + ( m2(lh) - m1(lh) ) .* h(lh) * 6;
  
  lh = h >= 1/6 & h <  1/2;
  v(lh) = m2(lh);
  
  lh = h >= 1/2 & h <  2/3;
  v(lh) = m1(lh) + ( m2(lh) - m1(lh) ) .* ( 2/3 - h(lh) ) * 6;   
  
  lh = h >= 2/3;
  v(lh) = m1(lh);

