function main()
% gui for quantum scattering

    % clear everything first
    clear all;
    close all;

    % get screensize and set geometries
    scnsize = get(0,'ScreenSize');
    height = 600;
    width = 800;
    pos = [scnsize(3)/2-width/2 scnsize(4)/2-height/2 width height];

    % initialize gui
    qs_main = figure('Visible','off',...            % hides the figure
               'Position',pos,...                   % sets position x/y and width/height
               'MenuBar','none',...                 % deactivates the menu bar
               'NumberTitle','off',...              % deactivates the 'figure n' in title
               'Name','Quantum Scattering');        % defines the title

    % guidata
    set(1,'Tag','QS_MAIN')
    myhandles = guihandles(qs_main);

    % some default values for gui widgets
    plot_width = 500;
    plot_height = 225;
    button_width = 70;
    button_height = 25;
    label_width = 75;
    text_width = 100;
    text_height = 18;
    bigmargin = 50;
    smallmargin = 5;
    
    % some variables
    myhandles.mode = 0; % mode: 0 for unset
                        %       1 for 1 dim
                        %       2 for 2 dim
    
    % 1 dim
    myhandles.kinenergy = 35;
    myhandles.potenergy = 30;
                        
    % 2 dim
    myhandles.numslits = 2;
    myhandles.slitwidth = 2/3;
    myhandles.slitdist = 1;
    myhandles.potheight = 10000000;
    myhandles.potwidth = 1/2;
    myhandles.closeoneslit = 1;
    myhandles.compressed = 0;


    % set handles in guidata
    guidata(qs_main,myhandles);

    % add main buttons (switch between 1 and 2 dim)
    myhandles.button1dim = uicontrol('Style','togglebutton',...
               'String','1 Dimension',...
               'Position',[width/2-120,height-button_height-smallmargin,110,button_height],...
               'Callback',{@button1dim_Callback});
    myhandles.button2dim = uicontrol('Style','togglebutton',...
               'String','2 Dimensions',...
               'Position',[width/2+10,height-button_height-smallmargin,110,button_height],...
               'Callback',{@button2dim_Callback});
    % add starting label
    myhandles.labelstart = uicontrol('Style','text',...
               'String','select a mode to start the quantum scattering',...
               'BackgroundColor',get(qs_main,'Color'),...
               'FontWeight','bold','FontSize',20,...
               'Position',[width/2-350,height/2-50,700,100]);
        

    % add general ui controls
    myhandles.buttongetdata = uicontrol('Style','pushbutton',...
               'String','Get Data',...
               'Position',[bigmargin+plot_width+bigmargin,2*bigmargin,button_width,button_height],...
               'Visible','off',...
               'Callback',{@buttongetdata_Callback});
    myhandles.buttonstart = uicontrol('Style','pushbutton',...
               'String','Start',...
               'Position',[bigmargin+plot_width+bigmargin+button_width+smallmargin,2*bigmargin,button_width,button_height],...
               'Visible','off',...
               'Enable','off',...
               'Callback',{@buttonstart_Callback});
    myhandles.labelsim = uicontrol('Style','text',...
               'Visible','off',...
               'Position',[2*smallmargin,2*smallmargin,plot_width+bigmargin,plot_width+bigmargin]);
    myhandles.labelsim2 = uicontrol('Style','text',...
               'Visible','off',...
               'String','no data loaded',...
               'FontWeight','bold',...
               'FontSize',18,...
               'Position',[2*smallmargin,2*smallmargin+(plot_width+bigmargin)/2,plot_width+bigmargin,50]);

    % add specific ui controls for 1 dim mode
    myhandles.axesqs2 = axes('Units','pixels',...
                             'Position',[bigmargin,bigmargin+plot_height+bigmargin,plot_width,plot_height],...
                             'Visible','off');
    myhandles.axesqs1 = axes('Units','pixels',...
                             'Position',[bigmargin,bigmargin,plot_width,plot_height],...
                             'Visible','off');
    myhandles.axesqs3 = axes('Visible','off');
    myhandles.lineqs1 = line('Visible','off',...
                             'Parent',myhandles.axesqs2);
    myhandles.lineqs2 = line('Visible','off',...
                             'Parent',myhandles.axesqs3);
    myhandles.lineqs3 = line('Visible','off',...
                             'Parent',myhandles.axesqs1);
    myhandles.flabel1 = uicontrol('Style','text',...
                                 'String','Kin. energy:',...
                                 'BackgroundColor',get(qs_main,'Color'),...
                                 'Position',[bigmargin*2+plot_width,height-3*bigmargin-2*button_height-2*smallmargin,label_width,text_height],...
                                 'Visible','off',...
                                 'HorizontalAlignment','left');
    myhandles.ftext1 = uicontrol('Style','edit',...
                                'Position',[bigmargin*2+plot_width+label_width+smallmargin,height-3*bigmargin-2*button_height-2*smallmargin,text_width,text_height],...
                                'Visible','off',...
                                'String',num2str(myhandles.kinenergy),...
                                'HorizontalAlignment','right');
    myhandles.flabel2 = uicontrol('Style','text',...
                                 'String','Pot height:',...
                                 'BackgroundColor',get(qs_main,'Color'),...
                                 'Position',[bigmargin*2+plot_width,height-3*bigmargin-2*button_height-3*smallmargin-text_height,label_width,text_height],...
                                 'Visible','off',...
                                 'HorizontalAlignment','left');
    myhandles.ftext2 = uicontrol('Style','edit',...
                                'Position',[bigmargin*2+plot_width+label_width+smallmargin,height-3*bigmargin-2*button_height-3*smallmargin-text_height,text_width,text_height],...
                                'Visible','off',...
                                'String',num2str(myhandles.potenergy),...
                                'HorizontalAlignment','right');
    
    
    % add specific ui controls for 2 dim mode    
    myhandles.axesqs4 = axes('Units','pixels',...
                             'Position',[7*smallmargin,7*smallmargin,plot_width,plot_height*2+bigmargin],...
                             'Visible','off');
    axis(myhandles.axesqs4,'off'); % disable axis 
    myhandles.imageqs1 = image('Parent',myhandles.axesqs4,...
                               'Visible','off');
    myhandles.imageqs2 = image('Parent',myhandles.axesqs4,...
                               'Visible','off');
    myhandles.label1 = uicontrol('Style','text',...
                                 'String','#Slits:',...
                                 'BackgroundColor',get(qs_main,'Color'),...
                                 'Position',[bigmargin*2+plot_width,height-3*bigmargin-2*button_height-2*smallmargin,label_width,text_height],...
                                 'Visible','off',...
                                 'HorizontalAlignment','left');
    myhandles.text1 = uicontrol('Style','edit',...
                                'Position',[bigmargin*2+plot_width+label_width+smallmargin,height-3*bigmargin-2*button_height-2*smallmargin,text_width,text_height],...
                                'Visible','off',...
                                'String',num2str(myhandles.numslits),...
                                'HorizontalAlignment','right');
    myhandles.label2 = uicontrol('Style','text',...
                                 'String','Slit width:',...
                                 'BackgroundColor',get(qs_main,'Color'),...
                                 'Position',[bigmargin*2+plot_width,height-3*bigmargin-2*button_height-3*smallmargin-text_height,label_width,text_height],...
                                 'Visible','off',...
                                 'HorizontalAlignment','left');
    myhandles.text2 = uicontrol('Style','edit',...
                                'Position',[bigmargin*2+plot_width+label_width+smallmargin,height-3*bigmargin-2*button_height-3*smallmargin-text_height,text_width,text_height],...
                                'Visible','off',...
                                'String',num2str(myhandles.slitwidth),...
                                'HorizontalAlignment','right');
    myhandles.label3 = uicontrol('Style','text',...
                                 'String','Slit dist:',...
                                 'BackgroundColor',get(qs_main,'Color'),...
                                 'Position',[bigmargin*2+plot_width,height-3*bigmargin-2*button_height-4*smallmargin-2*text_height,label_width,text_height],...
                                 'Visible','off',...
                                 'HorizontalAlignment','left');
    myhandles.text3 = uicontrol('Style','edit',...
                                'Position',[bigmargin*2+plot_width+label_width+smallmargin,height-3*bigmargin-2*button_height-4*smallmargin-2*text_height,text_width,text_height],...
                                'Visible','off',...
                                'String',num2str(myhandles.slitdist),...
                                'HorizontalAlignment','right');
    myhandles.label4 = uicontrol('Style','text',...
                                 'String','Pot height:',...
                                 'BackgroundColor',get(qs_main,'Color'),...
                                 'Position',[bigmargin*2+plot_width,height-3*bigmargin-2*button_height-5*smallmargin-3*text_height,label_width,text_height],...
                                 'Visible','off',...
                                 'HorizontalAlignment','left');
    myhandles.text4 = uicontrol('Style','edit',...
                                'Position',[bigmargin*2+plot_width+label_width+smallmargin,height-3*bigmargin-2*button_height-5*smallmargin-3*text_height,text_width,text_height],...
                                'Visible','off',...
                                'String',num2str(myhandles.potheight),...
                                'HorizontalAlignment','right');
    myhandles.label5 = uicontrol('Style','text',...
                                 'String','Pot width:',...
                                 'BackgroundColor',get(qs_main,'Color'),...
                                 'Position',[bigmargin*2+plot_width,height-3*bigmargin-2*button_height-6*smallmargin-4*text_height,label_width,text_height],...
                                 'Visible','off',...
                                 'HorizontalAlignment','left');
    myhandles.text5 = uicontrol('Style','edit',...
                                'Position',[bigmargin*2+plot_width+label_width+smallmargin,height-3*bigmargin-2*button_height-6*smallmargin-4*text_height,text_width,text_height],...
                                'Visible','off',...
                                'String',num2str(myhandles.potwidth),...
                                'HorizontalAlignment','right');
    myhandles.checkbox1 = uicontrol('Style','checkbox',...
                                'String','Close lower part',...
                                'BackgroundColor',get(qs_main,'Color'),...
                                'Position',[bigmargin*2+plot_width,height-3*bigmargin-2*button_height-7*smallmargin-5*text_height,text_width*2,text_height],...
                                'Visible','off',...
                                'Value',myhandles.closeoneslit,...
                                'HorizontalAlignment','left');
    myhandles.checkbox2 = uicontrol('Style','checkbox',...
                                'String','Compress data',...
                                'BackgroundColor',get(qs_main,'Color'),...
                                'Position',[bigmargin*2+plot_width,height-3*bigmargin-2*button_height-8*smallmargin-6*text_height,text_width*2,text_height],...
                                'Visible','off',...
                                'Value',myhandles.compressed,...
                                'HorizontalAlignment','left');
    

    % normalize everything in case of resize and set parent
    set([myhandles.button1dim,...
        myhandles.button2dim,...
        myhandles.labelstart,...
        myhandles.buttongetdata,...
        myhandles.labelsim,...
        myhandles.labelsim2,...
        myhandles.buttonstart,...
        myhandles.axesqs1,...
        myhandles.axesqs2,...
        myhandles.axesqs3,...
        myhandles.axesqs4,...
        myhandles.label1,...
        myhandles.label2,...
        myhandles.label3,...
        myhandles.label4,...
        myhandles.label5,...
        myhandles.text1,...
        myhandles.text2,...
        myhandles.text3,...
        myhandles.text4,...
        myhandles.text5,...
        myhandles.checkbox1,...
        myhandles.checkbox2],...
        'Units','normalized',...
        'Parent',qs_main);

                        
    % Save the structure
    guidata(qs_main,myhandles)

    % show
    set(qs_main,'Visible','On');

    
    % pushbutton callbacks
    function button1dim_Callback(~,~)
        % first time?
        if (strcmp(get(myhandles.labelstart,'Visible'),'on'))
            show_main_controls();
        end
        
        % set mode
        set_mode(1);
        
    end
    function button2dim_Callback(~,~)
        % first time?
        if (strcmp(get(myhandles.labelstart,'Visible'),'on'))
            show_main_controls();
        end
        
        % set mode
        set_mode(2);
        
    end
    function buttongetdata_Callback(~,~)
        myhandles = guidata(qs_main);
        if (myhandles.mode == 1)
            % get data
            get1dimdata();
            Quantenstreuung1D();

            % enable axis again
            axis([myhandles.axesqs1,myhandles.axesqs2],'on');

            % set data for graphic output
            set1dimdata();
            
            % enable start and hide label
            set(myhandles.buttonstart,'Enable','on');
            set(myhandles.labelsim,'Visible','off');
            set(myhandles.labelsim2,'Visible','off');
        elseif (myhandles.mode == 2)
            % get data
            get2dimdata();
            Quantenstreuung2D();
            
            % set data for graphic output
            set2dimdata();

            % enable start
            set(myhandles.buttonstart,'Enable','on');
            set(myhandles.labelsim,'Visible','off');
            set(myhandles.labelsim2,'Visible','off');
        else
            error('mode not specified');
        end
    end
    function buttonstart_Callback(~,~)
        if (myhandles.mode == 1)
            play_qm1D();
        elseif (myhandles.mode == 2)
            play_qm2D();
        end
    end
         
end

function show_main_controls()
   
    % get gui data
    qs_main = findobj(0,'Tag','QS_MAIN');
    myhandles = guidata(qs_main);
    
    % hide start label
    set(myhandles.labelstart,'Visible','off');
    
    % show main controls
    set(myhandles.buttongetdata,'Visible','on');
    set(myhandles.buttonstart,'Visible','on');

end

function set_mode(mode)

    % get gui data
    qs_main = findobj(0,'Tag','QS_MAIN');
    myhandles = guidata(qs_main);

    % if other button is toggled, clear and enable it and disable current button
    if (mode == 1)
        set(myhandles.button2dim,'Value',0);
        set(myhandles.button2dim,'Enable','on');
        set(myhandles.button1dim,'Enable','off');
        
        % change title
        set(qs_main,'Name','Quantum Scattering 1 Dimension');
    elseif (mode == 2)
        set(myhandles.button1dim,'Value',0);
        set(myhandles.button1dim,'Enable','on');
        set(myhandles.button2dim,'Enable','off');

        % change title
        set(qs_main,'Name','Quantum Scattering 2 Dimensions');      
    end

    % set mode
    myhandles.mode = mode;

    % disable start
    set(myhandles.buttonstart,'Enable','off');

    % show ui controls which are needed
    if (mode == 1)
        set(myhandles.axesqs1,'Visible','on');
        set(myhandles.axesqs2,'Visible','on');
        axis([myhandles.axesqs1,myhandles.axesqs2],'off'); % disable axis
        set([myhandles.flabel1,...
             myhandles.flabel2,...
             myhandles.ftext1,...
             myhandles.ftext2],...
             'Visible','on');
    elseif (mode == 2)
        set(myhandles.axesqs4,'Visible','on');
        axis(myhandles.axesqs4,'off'); % disable axis
        set([myhandles.label1,...
             myhandles.label2,...
             myhandles.label3,...
             myhandles.label4,...
             myhandles.label5,...
             myhandles.text1,...
             myhandles.text2,...
             myhandles.text3,...
             myhandles.text4,...
             myhandles.text5,...
             myhandles.checkbox1,...
             myhandles.checkbox2],...
             'Visible','on');
    end

    % hide ui controls from other mode
    if (mode == 1)
        set(myhandles.axesqs4,'Visible','off');
        set([myhandles.label1,...
             myhandles.label2,...
             myhandles.label3,...
             myhandles.label4,...
             myhandles.label5,...
             myhandles.text1,...
             myhandles.text2,...
             myhandles.text3,...
             myhandles.text4,...
             myhandles.text5,...
             myhandles.checkbox1,...
             myhandles.checkbox2],...
             'Visible','off');
    elseif (mode == 2)
        set(myhandles.axesqs1,'Visible','off');
        set(myhandles.axesqs2,'Visible','off');
        set([myhandles.flabel1,...
             myhandles.flabel2,...
             myhandles.ftext1,...
             myhandles.ftext2],...
             'Visible','off');
    end
    
    % hide everything else
    set([myhandles.axesqs3,...
         myhandles.lineqs1,...
         myhandles.lineqs2,...
         myhandles.lineqs3,...
         myhandles.imageqs1,...
         myhandles.imageqs2],...
         'Visible','off');

    % show information that no data is loaded yet
    set(myhandles.labelsim,'Visible','on');
    set(myhandles.labelsim2,'Visible','on');

    % save
    guidata(qs_main,myhandles);
    
end

function get1dimdata()

    % get guidata
    qs_main = findobj(0,'Tag','QS_MAIN');
    myhandles = guidata(qs_main);
   
    myhandles.kinenergy = str2double(get(myhandles.ftext1,'String'));
    myhandles.potenergy = str2double(get(myhandles.ftext2,'String'));
   
    % save guidata
    guidata(qs_main,myhandles);
    
end

function get2dimdata()

    % get guidata
    qs_main = findobj(0,'Tag','QS_MAIN');
    myhandles = guidata(qs_main);
   
    myhandles.numslits = str2double(get(myhandles.text1,'String'));
    myhandles.slitwidth = str2double(get(myhandles.text2,'String'));
    myhandles.slitdist = str2double(get(myhandles.text3,'String'));
    myhandles.potheight = str2double(get(myhandles.text4,'String'));
    myhandles.potwidth = str2double(get(myhandles.text5,'String'));
    myhandles.closeoneslit = get(myhandles.checkbox1,'Value');
   
    % save guidata
    guidata(qs_main,myhandles);
    
end

function set1dimdata()

    % get gui data
    qs_main = findobj(0,'Tag','QS_MAIN');
    myhandles = guidata(qs_main);
    
    AD = myhandles.animation_data;

    set(myhandles.axesqs2,'YLim',[0 3* max(AD.potential(:))]);   % Setting an axis-limit (y-axis); potential is 1/3*height of plot 
    set(myhandles.lineqs1,'XData',[-AD.grid_range,0,0,AD.grid_range],...
                          'YData',[0,0,1,1]*max(AD.potential(:)),...
                          'Color','k',...
                          'Visible','on'); % Drawing a line representig the potential


    pos = get(myhandles.axesqs2,'Position');                % getting position of first axes and writing it to the variable pos
    pos(2) = pos(2) + 1/3*pos(4)*(AD.energy_relation);   % shifting the bottom line considering the relation between potential and kinetic energy
    pos(4) = pos(4) - 1/3*pos(4)*(AD.energy_relation);   % shrinking the height of the second axes, so that the upper edges of the first and second axes are meeting

    set(myhandles.axesqs3,'Position',pos, 'YLim',[0 max(AD.abs_val_wavefun(:))],'Visible','on',...
                'XTick',[],'YAxisLocation','right','XAxisLocation','top','Color','none'); % a second system of axes is initialised by using pos;
    MaxHeight = 1 / AD.step_width;             % maximum height the probability density can reach, if the possibility to find the particle is 100% in one space intervall
    MaxProb   = max(AD.abs_val_wavefun(:)) / MaxHeight;  % calculating the maximum of the probability density (considering all times)
    ProbStep  = MaxProb / length(get(myhandles.axesqs4,'YTick'));
    LabelVec  = round((0:ProbStep:MaxProb)*1e2);
    set(myhandles.axesqs3,'YTickLabel',LabelVec);
    set(myhandles.lineqs2,'XData',AD.x,...
                          'YData',AD.abs_val_wavefun(1,:),...
                          'Color','blue',...
                          'Visible','on'); % the squared absolute value of the wavefunction is drawn into this axes


    set(myhandles.axesqs1,'XLim',[min(AD.momentum),max(AD.momentum)],'YLim',[0 max(AD.abs_val_momentumwavefun(:))]);
    set(myhandles.lineqs3,'XData',AD.momentum,...
                          'YData',AD.abs_val_momentumwavefun(1,:),...
                          'Color','red',...
                          'Visible','on');
                      
end

function set2dimdata()

    % get gui data
    qs_main = findobj(0,'Tag','QS_MAIN');
    myhandles = guidata(qs_main);
    
    AD = myhandles.animation_data;

    abs_val_wavefunction = (conj(AD.wavefunction).*AD.wavefunction);
    norm_factor = max(abs_val_wavefunction(:));
    saturation_modifier = 0.7;
    clear abs_val_wavefunction;

    % analysing the start values (help analyse_complex for further information)
    [phas_wavefunction, abs_val_wavefunction] = analyse_complex(AD.start_wavefunction , norm_factor);
    % taking the square root of abs_val_wavefunction to get better visualisation
    abs_val_wavefunction = abs_val_wavefunction .^ (1/2);

    % results are seen as hls values hue = phase, lightness = abs_val
    % saturation = arbitrary then they are converted into a rgb colormap
    % only this values can be visualised with matlab
    myhandles.hls_mat = ones(AD.nr_of_gridpoints , AD.nr_of_gridpoints ,3) * saturation_modifier;
    myhandles.hls_mat(:,:,1) = phas_wavefunction(:,:);
    myhandles.hls_mat(:,:,2) = abs_val_wavefunction(:,:);
    rgb_mat = hls2rgb(myhandles.hls_mat);

    % plotting the start values
    set(myhandles.imageqs1,'CData',rgb_mat(:,:,:),...
                           'Visible','on');
    color_vector = zeros(1,1,3);
    color_vector(1,1,:) = [1,1,1];
    color_vector = repmat(color_vector,AD.nr_of_gridpoints,AD.nr_of_gridpoints);
    set(myhandles.imageqs2,'CData',color_vector,...
                           'AlphaData',AD.potential / max(AD.potential(:))/3,...
                           'Visible','on');
    axis image;
    drawnow

    % save guidata
    guidata(qs_main,myhandles);
    
end

function play_qm1D()

    % get gui data
    qs_main = findobj(0,'Tag','QS_MAIN');
    myhandles = guidata(qs_main);

    for i = 1:size(myhandles.animation_data.abs_val_wavefun,1);                          % this for-loop performs the time evolution
      set(myhandles.lineqs2,'YData',myhandles.animation_data.abs_val_wavefun(i,:))          % merely updating the y-data increases computing; drawing 
      set(myhandles.lineqs3,'YData',myhandles.animation_data.abs_val_momentumwavefun(i,:))  % a new plot for every timestep is unnecessary  
      drawnow;
    end
    
end

function play_qm2D()

    % get gui data
    qs_main = findobj(0,'Tag','QS_MAIN');
    myhandles = guidata(qs_main);

    compressed=myhandles.compressed;

    abs_val_wavefunction = (conj(myhandles.animation_data.wavefunction).*myhandles.animation_data.wavefunction);
    norm_factor = max(abs_val_wavefunction(:));
    
    % analysing all timesteps
    [phas_wavefunction, abs_val_wavefunction] =  analyse_complex(myhandles.animation_data.wavefunction , norm_factor);

    % taking the square root of abs_val_wavefunction to get better visualisation
    abs_val_wavefunction = abs_val_wavefunction .^ (1/2);
    % if the optimised method is used a rgb matrix
    % containing the changing rgb values is calculated
    if compressed ~=0;
      compressed_hls_mat = ones(size(phas_wavefunction,1),size(phas_wavefunction,2),3) * saturation_modifier;
      compressed_hls_mat(:,:,1) = phas_wavefunction;
      compressed_hls_mat(:,:,2) = abs_val_wavefunction;
      compressed_rgb_mat = hls2rgb(compressed_hls_mat);
      start_index = 0;
    end
    
    % animation: the two methods must be handled differently
    %  uncompressed: for every timestep the whole rgb-matrix is calculated
    %  compressed:   only the data of compressed_rgb_mat is written into rgb_mat
    for inc=1:1:myhandles.animation_data.nr_of_timesteps    
      if compressed == 0  
        myhandles.hls_mat(:,:,1) = phas_wavefunction(:,:,inc);
        myhandles.hls_mat(:,:,2) = abs_val_wavefunction(:,:,inc);
        rgb_mat = hls2rgb(myhandles.hls_mat);
      elseif compressed ~= 0
        temp_index = myhandles.animation_data.index(start_index + 1 : myhandles.animation_data.index_to_index(inc));  
        rgb_mat(temp_index)                         = compressed_rgb_mat(1, start_index + 1 : myhandles.animation_data.index_to_index(inc) ,1);
        rgb_mat(temp_index +     size(rgb_mat,1)^2) = compressed_rgb_mat(1, start_index + 1 : myhandles.animation_data.index_to_index(inc) ,2);
        rgb_mat(temp_index + 2 * size(rgb_mat,1)^2) = compressed_rgb_mat(1, start_index + 1 : myhandles.animation_data.index_to_index(inc) ,3);
        start_index = myhandles.animation_data.index_to_index(inc);
      end
      set(myhandles.imageqs1,'CData',rgb_mat(:,:,:));
      drawnow
    end
end
